//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __Sample__
#define __Sample__

#include "CUtils.h"
#include "TMath.h"

#define NORDER 20
#define NORDERHP 12

using namespace std;

/**
 * Produce a filter defined by a set of zeros, poles and a gain factor.
 * This class was designed to create and use a filter defined by zeros and poles. This class is private and can only be used by its friends.
 *
 * \author    Florent Robinet
 */
class ZPGFilter {

 public:
  friend class Sample;  ///< friendly class
  friend class IIRFilter;///< friendly class

 private:

  ZPGFilter(const unsigned int numZeros, const unsigned int numPoles);
  virtual ~ZPGFilter(void);

  bool WToZ(void);     ///< transform to the z-plane
  
  unsigned int nzeros; ///< number of zeros
  unsigned int npoles; ///< number of poles
  double *zeros[2];    ///< complex zeros
  double *poles[2];    ///< complex poles
  double gain[2];      ///< complex gain
};


/**
 * Produce an infinite impulse response filter.
 * This class is private and can only be used by its friends.
 *
 * \author    Florent Robinet
 */
class IIRFilter {

 public:
  friend class Sample;           ///< friendly class

 private:

  IIRFilter(ZPGFilter *zpgFilter);///< constructor
  virtual ~IIRFilter(void);       ///< destructor

  unsigned int nzeros;            ///< number of zeros
  unsigned int npoles;            ///< number of poles
  double *zeros[2];               ///< zeros (complex)
  double *poles[2];               ///< poles (complex)
  double gain[2];                 ///< gain (complex)
  unsigned int numDirect;         ///< The number of direct filter coefficients.
  unsigned int numRecurs;         ///< The number of recursive filter coefficients.
  double *directCoef;             ///< The direct filter coefficients.
  double *recursCoef;             ///< The recursive filter coefficients.
  double *history;                ///< The previous values of w.
  double *history_init;           ///< The initial values of w.

};


/**
 * Transform a discrete time series.
 * It includes:
 * - DC removal
 * - Tukey windowing
 * - resampling
 * - frequency high-passing
 *
 * \author    Florent Robinet
 */
class Sample {

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the Sample class.
   * @param aVerbosity verbosity level
   */
  Sample(const int aVerbosity=0);

  /**
   * Destructor of the Sample class.
   */
  virtual ~Sample(void);
  /**
     @}
  */

  /**
   * Activates/Deactivates the DC removal.
   * The DC component is computed and is subtracted from all data samples. The final vector has a DC component equal to 0.
   * @param aDCremove activate/deactivate the DC removal
   */
  void SetDCRemoval(const bool aDCremove=true){ fRemoveDC=aDCremove; };

  /**
   * Defines windowing properties.
   * It is possible to window the input data.
   * A Tukey window is defined by a 'Tukey fraction'. To make it clear, let's take the example of a 10% Tukey fraction. This means that the first 5% of the input data vector will be smoothly transition from 0 to 1. Similarly, the last 5% of the input data vector will be smoothly transition from 1 to 0.
   * @param aTukeyFraction tukey fraction between 0 and 1
   */
  bool SetWindows(const double aTukeyFraction);

  /**
   * Defines frequencies for the transformation.
   * It is possible to work with a sampling frequency smaller than the native frequency, often for performance reasons. It must be an integer value and a power of 2. When transformed (see Transform()), the input data are first downsampled to the working frequency with an anti-aliasing filter.
   *
   * In a second step, it is possible to highpass the data. When transformed (see Transform()), the input data are highpass-filtered.
   * @param aNativeFrequency native sampling frequency [Hz]
   * @param aWorkingFrequency working sampling frequency [Hz]
   * @param aHighPassFrequency highpass frequency [Hz]
   */
  bool SetFrequencies(const int aNativeFrequency, const int aWorkingFrequency, const double aHighPassFrequency=0.0);

  /**
   * Sets a new native sampling frequency.
   * This function can only be called if SetFrequencies() has been previously called and if the new working sampling frequency is compatible with previous frequency settings.
   * @param aNativeFrequency new native sampling frequency [Hz]
   */
  bool SetNativeFrequency(const int aNativeFrequency);

  /**
   * Sets a new working sampling frequency.
   * It is possible to work with a sampling frequency smaller than the native frequency, often for performance reasons. This function sets a new working frequency. It must be an integer value and a power of 2. When transformed (see Transform()), the input data are first downsampled to the working frequency with an anti-aliasing filter.
   *
   * This function can only be called if SetFrequencies() has been previously called and if the new working sampling frequency is compatible with previous frequency settings.
   * @param aWorkingFrequency new working sampling frequency [Hz]
   */
  bool SetWorkingFrequency(const int aWorkingFrequency);

  /**
   * Defines a highpass frequency.
   * It is possible to highpass the data. This function sets a new highpass frequency. When transformed (see Transform()), the input data are filtered.
   * 
   * This function can only be called if SetFrequencies() has been previously called and if the new highpass frequency is compatible with previous frequency settings.
   * @param aHighPassFrequency highpass frequency [Hz]
   */
  bool SetHighPassFrequency(const double aHighPassFrequency);

  /**
   * Transforms data.
   * The input data vector (time-domain) is transformed as previously defined: see SetFrequencies(). The input vector must have an integer duration. The memory for the output vector must be allocated with the right size before calling this function. No size check will be performed and the data size provided by the user will be trusted.
   *
   * IMPORTANT: the input vector can be altered in case of presampling. Do not use after calling this function.
   * @param invectorsize input vector size (in samples)
   * @param invector input data vector (time-domain)
   * @param outvectorsize output vector size (in samples)
   * @param outvector output data vector (time-domain)
   */
  bool Transform(const int invectorsize, double *invector, const int outvectorsize, double *outvector);

  /**
   * Returns the current native sampling frequency.
   */
  inline int GetNativeFrequency(void){ return fNativeFrequency; };

  /**
   * Returns the current working sampling frequency.
   */
  inline int GetWorkingFrequency(void){ return fWorkingFrequency; };

  /**
   * Returns the current highpass frequency.
   */
  inline double GetHighPassFrequency(void){ return fHighPassFrequency; };

  /**
   * Returns the current Tukey fraction.
   */
  inline double GetTukeyFraction(void){ return fTukeyFraction; };


 protected:

  bool fRemoveDC;              ///< remove DC
  double fTukeyFraction;       ///< window fraction
  int fWorkingFrequency;       ///< working sampling frequency
  int fNativeFrequency;        ///< native sampling frequency
  int fVerbosity;              ///< verbosity level
  double fHighPassFrequency;   ///< cutoff frequency

 private:

  // RESAMPLING
  bool MakeSamplingFilters(void);  ///< create sampling filters
  bool Resample(int invectorsize, double *invector, double *outvector);
  bool presample;                  ///< pre-sample flag
  int fResampleFactor;             ///< resampling factor (>=1)
  int fIntermediateFrequency;      ///< intermediate sampling frequency
  IIRFilter *iirFilter[NORDER];    ///< downsampling filters
  bool PreSample(int &datavectorsize, double *datavector); ///< presampling
  
  
  // HIGHPASS
  bool MakeHighPassFilters(void);  ///< create highpass filters
  bool HighPass(const int invectorsize, double *invector);
  IIRFilter *iirFilterHP[NORDERHP];///< highpass filters

  // FILTERING
  bool ApplyFilter(const int datavectorsize, double *datavector, IIRFilter *filter); ///< filtering

  // WINDOWING
  bool ApplyTukeyWindow(const int datavectorsize, double *datavector);

  // DC REMOVAL
  void RemoveDC(const int datavectorsize, double *datavector);
  
  ClassDef(Sample,0)  

};

#endif


