//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __ReadTriggerSegments__
#define __ReadTriggerSegments__

#include "TROOT.h"
#include "TChain.h"
#include "Segments.h"

using namespace std;

/**
 * Access segments in trigger files.
 * This class is a sub-product of the ReadTriggers class. It should be used when you're only interested by the segment content of trigger files. Only the segment content of the files is loaded so this class is much faster than the ReadTriggers class since the triggers are never loaded. This is quite useful when hundreds of files need to be scanned only to access the timing information.
 *
 * IMPORTANT: the input files in 'apattern' should be time-sorted and with NO overlap
 *
 * IMPORTANT_2: the segments in the trigger files must follow the <a href="../../Main/convention.html#triggers">GWOLLUM convention</a> for triggers.
 * \author    Florent Robinet
 */
class ReadTriggerSegments{

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the ReadTriggerSegments class.
   * The segments in files designated by the file pattern 'aPattern' are loaded. The segment tree is accessed in the root directory 'aDirectory'. The verbosity level, set by 'aVerbose', follows the <a href="../../Main/convention.html#verbosity">GWOLLUM convention</a> for verbosity.
   *
   * IMPORTANT: the input files in 'aPattern' should be time-sorted and with NO overlap
   * @param aPattern input file pattern
   * @param aDirectory trigger ROOT directory
   * @param aVerbose verbosity level
   */
  ReadTriggerSegments(const string aPattern, const string aDirectory="", const int aVerbose=0);

  /**
   * Destructor of the ReadTriggerSegments class.
   */
  virtual ~ReadTriggerSegments(void);
  /**
     @}
  */

  /**
   * Returns the list of trigger segments.
   * DO NOT DELETE OR MODIFY
   */
  inline Segments* GetSegments(void){return Seg;};

  /**
   * Returns the starting time.
   */
  inline double GetTimeMin(void){return Seg->GetStart(0);};

  /**
   * Returns the ending time.
   */
  inline double GetTimeMax(void){return Seg->GetEnd(Seg->GetNsegments()-1);};
  
  /**
   * Returns the list of input files which contain data in a given time range.
   * This list is a string where file paths are separated by a white space.
   * If no file is found, "none" is returned.
   * @param aTimeMin starting time
   * @param aTimeMax ending time
   */
  string GetTriggerFiles(const double aTimeMin, const double aTimeMax);

  /**
   * Returns the list of input files which contain data in a given segment list.
   * The returned list is a string where file paths are separated by a white space.
   * If no file is found, "none" is returned.
   * @param aSegSelect segment list
   */
  string GetTriggerFiles(Segments *aSegSelect);

  /**
   * Returns the number of input segments trees.
   */
  inline int GetNTrees(void){ return (int)triggerfilename.size(); };

  /**
   * Returns the number of input files.
   */
  inline int GetNFiles(void){ return (int)triggerfilename.size(); };

  /**
   * Returns the list of file time starts.
   * DO NOT DELETE OR MODIFY
   */
  inline double* GetFileStarts(void){ return Tstart; };

  /**
   * Returns the list of file time stops.
   * DO NOT DELETE OR MODIFY
   */
  inline double* GetFileStops(void){ return Tstop; };

  /**
   * Returns the list of file time starts.
   * DO NOT DELETE OR MODIFY
   */
  inline double* GetTreeStarts(void){ return Tstart; };

  /**
   * Returns the list of file time stops.
   * DO NOT DELETE OR MODIFY
   */
  inline double* GetTreeStops(void){ return Tstop; };

  /**
   * Sets verbosity level.
   * @param averbose new verbosity level
   */
  inline void SetVerbose(const int aVerbose){ Verbose=aVerbose; };

  /**
   * Returns the general status of the ReadTriggerSegments object.
   */
  inline bool GetStatus(void){ return status_OK; };

  /**
   * Returns a unique ID for this trigger set.
   */
  inline int GetTriggerID(void){ return randid; };

  /**
   * Returns a unique ID for this trigger set.
   */
  inline string GetTriggerStringID(void){ return srandid; };

 protected:

  // PARAMETERS
  bool status_OK;         ///< status
  int Verbose;            ///< verbosity level
  string Pattern;         ///< pattern of input root files
  string Dir;             ///< directory
  int randid;             ///< random integer id
  string srandid;         ///< random string id

  // SEGMENTS
  Segments *Seg;          ///< trigger segments 
  double   *Tstart;       ///< file starts (min start)
  double   *Tstop;        ///< file stops (max end)
  vector <string> triggerfilename; ///< root file names (paths)
  
 private:
  bool GetInputSegments(void); ///< get input segments from files

  ClassDef(ReadTriggerSegments,0)  
};

#endif


