//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "TriggerBuffer.h"

ClassImp(TriggerBuffer)

////////////////////////////////////////////////////////////////////////////////////
TriggerBuffer::TriggerBuffer(const int aSize, const string aStreamName, const int aVerbose): MakeTriggers(aStreamName, aVerbose) { 
////////////////////////////////////////////////////////////////////////////////////

  // allocate memory for the buffer
  BufferSize=aSize;
  b_time     = new double [BufferSize];
  b_freq     = new double [BufferSize];
  b_tstart   = new double [BufferSize];
  b_tend     = new double [BufferSize];
  b_fstart   = new double [BufferSize];
  b_fend     = new double [BufferSize];
  b_snr      = new double [BufferSize];
  b_q        = new double [BufferSize];
  b_amp      = new double [BufferSize];
  b_ph       = new double [BufferSize];
  b_pos      = 0;
  b_seg      = new Segments();
}

////////////////////////////////////////////////////////////////////////////////////
TriggerBuffer::~TriggerBuffer(void){
////////////////////////////////////////////////////////////////////////////////////
  if(Verbose>1) cout<<"TriggerBuffer::~TriggerBuffer"<<endl;
  ResetBuffer();
  delete b_time;
  delete b_freq;
  delete b_tstart;
  delete b_tend;
  delete b_fstart;
  delete b_fend;
  delete b_q;
  delete b_snr;
  delete b_amp;
  delete b_ph;
  delete b_seg;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerBuffer::FlushBuffer(void){
////////////////////////////////////////////////////////////////////////////////////

  // flush triggers
  for(int t=0; t<b_pos; t++){
    if(!MakeTriggers::AddTrigger(b_time[t], b_freq[t], b_snr[t], b_q[t], b_tstart[t], b_tend[t], b_fstart[t], b_fend[t], b_amp[t], b_ph[t])){
      return false;
    }
  }

  // flush segments
  if(this->Segments::GetNsegments()&&b_seg->GetStart(0)>=this->Segments::GetStart(this->Segments::GetNsegments()-1))// after last segments
    this->Segments::Append(b_seg);
  else
    for(int s=0; s<b_seg->GetNsegments(); s++) this->Segments::AddSegment(b_seg->GetStart(s),b_seg->GetEnd(s));

  
  // reset buffer
  ResetBuffer();
  
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerBuffer::SetBufferSegments(Segments *aSeg){
////////////////////////////////////////////////////////////////////////////////////
  if(s_end.size()&&aSeg->GetStart(0)<s_end.back()){
    cerr<<"TriggerBuffer::SetBufferSegments: segments must be chronological and with no overlap"<<endl;
    return false;
  }

  // save start - end
  s_start.push_back(aSeg->GetStart(0));
  s_end.push_back(aSeg->GetEnd(aSeg->GetNsegments()-1));

  // current index in the trigger buffer
  s_pos.push_back(b_pos);

  // update buffer livetime
  b_seg->Append(aSeg);
  
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerBuffer::AddTrigger(const double aTime,   const double aFrequency,
			       const double aSNR,    const double aQ,
			       const double aTstart, const double aTend,
			       const double aFstart, const double aFend,
			       const double aAmplitude, const double aPhase){
////////////////////////////////////////////////////////////////////////////////////
  if(!BufferSize) return Triggers::AddTrigger(aTime,aFrequency,aSNR,aQ,aTstart,aTend,aFstart,aFend,aAmplitude,aPhase);
  
  if(b_pos==BufferSize){
    cerr<<"TriggerBuffer::AddTrigger: buffer is full ("<<BufferSize<<" triggers)"<<endl;
    return false;
  }

  b_time[b_pos]   = aTime;
  b_freq[b_pos]   = aFrequency;
  b_tstart[b_pos] = aTstart;
  b_tend[b_pos]   = aTend;
  b_fstart[b_pos] = aFstart;
  b_fend[b_pos]   = aFend;
  b_q[b_pos]      = aQ;
  b_snr[b_pos]    = aSNR;
  b_amp[b_pos]    = aAmplitude;
  b_ph[b_pos]     = aPhase;
  b_pos++;
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerBuffer::ResetAfter(const double aTime){
////////////////////////////////////////////////////////////////////////////////////
  if(!b_seg->GetLiveTime()) return; // nothing to do
    
  // rewind buffer index and remove segments ending after input time
  for(int s=(int)s_end.size()-1; s>=0; s--){
    if(s_end[s]>aTime){
      b_pos=s_pos[s];
      s_start.pop_back();
      s_end.pop_back();
      s_pos.pop_back();
      continue;
    }
    break;
  }

  // remove segments
  if(s_end.size()) b_seg->TruncateAfter(s_end.back());
  else b_seg->Reset();
      
  return;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerBuffer::ResetBuffer(void){
////////////////////////////////////////////////////////////////////////////////////
  b_pos=0;
  s_start.clear();
  s_end.clear();
  s_pos.clear();
  b_seg->Reset();
  
  return;
}

////////////////////////////////////////////////////////////////////////////////////
Segments* TriggerBuffer::GetLastTriggerSegments(TH1D *aThr, const double aInfValue){ 
////////////////////////////////////////////////////////////////////////////////////

  Segments *S = new Segments();
  if(!s_start.size()) return S;

  if(aThr==NULL){
    for(int t=s_pos.back(); t<b_pos; t++){
      S->AddSegment(b_tstart[t],b_tend[t]);
    }
  }
  else{
    double fmin = aThr->GetXaxis()->GetXmin();
    double fmax = aThr->GetXaxis()->GetXmax();
    int binn;
    for(int t=s_pos.back(); t<b_pos; t++){
      if(b_freq[t]<fmin) continue;
      if(b_freq[t]>=fmax) continue;
      binn = aThr->GetXaxis()->FindBin(b_freq[t]); // get frequency bin
      if(aThr->GetBinContent(binn)>=aInfValue) continue; // infinity threshold
      if(b_snr[t]<aThr->GetBinContent(binn)) continue; // below threshold
      S->AddSegment(b_tstart[t],b_tend[t]);
    }
  }

  return S;
}
